body {
	background-color: #111;
	color: #777;
}
#Container {
color:white;
}

#Content{
	border-top: 1px dotted #222;


}

.left-border {
border-left: 1px dotted #222;
}
.right-border {
border-right: 1px dotted #222;
}

#Footer{
border-top: 1px dotted #222;
}

#progressBar {
  background-color: #fff;
  border: 1px solid #000;
}
#progressBartop {
  background-color: #fff;
  border: 1px solid #ccc;
}
#percentageBar {
  background-color: #4B4545;
}


h1{
background-color:#4B4545;
color:#fff;
}

#title{
color:#fff;
}


a, a:visited {
color:#888;
}
a.config{
color:#666;
}
a:hover{
color:blue;
}

.paused{background-color:red;padding:2px;color:black;}
.downloading{background-color:#00FF00;padding:2px;color:black;}
.idle{background-color:yellow;padding:2px;color:black;}
.spacer{color:white;margin-top:100px;margin-bottom:100px;}

#feed{color:white;}



hr
{
color:#222;
background-color:#222;
}

th{background-color:#000;color:white;}
tr{background-color:#222;}

td{border:1px solid #333 !important;}
table{border-spacing:0;}

.finished{background-color:#ff5151; color:black;}
.active{background-color:#4aff5f; color:black;}
.waiting{background-color:#fff44a; color:black;}



input, select, option {
background-color:#232323;
border-color:#3a3a3a;
color:white;
}

tr.High {background-color:#111;}
tr.Normal {background-color:#222;}
tr.Low {background-color:#333;}

legend{color:white}
.job_progress{background-color: #444;}

.pattern, .result{background-color:#111;}

a.unselected {
    border: 1px solid #C4C4C4;
    color:#C4C4C4;
}
span.unselected {
    border: 1px solid #555;
    color:#555;
}
.selected, a.unselected:hover {
    color: white;
    background-color:#333;
    border: 1px solid #555;
}
