body {
	background-color: #fff;
	color: #000;
}
#Container {
color:black;
}

#Content{
	border-top: 1px dotted #ccc;


}

.left-border {
border-left: 1px dotted #ccc;
}
.right-border {
border-right: 1px dotted #ccc;
}

#Footer{
border-top: 1px dotted #ccc;
}

#progressBar {
  background-color: #fff; 
  border: 1px solid #000; 
}
#progressBartop {
  background-color: #fff; 
  border: 1px solid #ccc; 
}
#percentageBar {
  background-color: #4B4545; 
}


h1{
background-color:#4B4545;
color:#fff;
}

#title{
color:#fff;
}

a:visited{
color:#000;

}
a {
color:black;
}
a.config{
color:#4B4545;
}
a:hover{
color:red;
}

.paused{background-color:red;padding:2px;}
.downloading{background-color:#00FF00;padding:2px;}
.idle{background-color:yellow;padding:2px;}
.spacer{color:white;margin-top:100px;margin-bottom:100px;}

#feed{color:white;}



hr
{
color:#ccc;
background-color:#ccc;
}

th{background-color:#4B4545;color:white;}
tr{background-color:#f5f5f5;}

td{border:1px solid #eeeeee !important;}
table{border-spacing:0;}

.finished{background-color:#ffd9d9;}
.active{background-color:#deffd9;}
.waiting{background-color:#ffffd9;}


tr.High {background-color:#ececec;}
tr.Normal {background-color:#f5f5f5;}
tr.Low {background-color:#fafafa;}

a.unselected {
    padding:1px 5px 1px 5px;
    border: 1px solid #555;
    color:#555;
    text-decoration:none;
}
span.unselected {
    padding:1px 5px 1px 5px;
    border: 1px solid #C4C4C4;
    color:#C4C4C4;
}
.selected, a.unselected:hover {
    padding:1px 5px 1px 5px;
    color: white;
    background-color:#333;
    border: 1px solid #555;
}