body {
    font-family: "Helvetica Neue", Helvetica, Arial, sans-serif !important;
    background-color: #E4E4E4;
    text-align: center;
}
.container {
    max-width: 1000px;
    border: 1px solid rgba(0, 0, 0, 0.2);
    background-color: #fff;
    text-align: left;
}
#inner {
    background-color: #fff;
    padding: 4px 32px 20px 32px;
    /*margin:5px;*/
}
#content {
    min-height: 550px;
}
#logo {
    float: center;
    /*margin-top:30px;*/
    
    z-index: 40;
    margin-bottom: -40px;
}
.language {
    min-width: 150px;
    width: 18%;
    padding: 1em 0em;
    margin: 5px 5px;
    text-align: center;
    float: left;
}
.language:hover {
    background-color: #E4E4E4;
}
.float {
    float: left;
}
.clear {
    clear: left;
}
.spacer {
    clear: both;
}
.inline {
    display: inline;
}
.quoteBlock {
    padding: 8px;
    border: 1px solid #ccc;
    margin-top: 1px;
    background-color: #f5f5f5;
    cursor: default;
}
.quoteBlock a {
    color: blue !important;
}
.success {
    color: #00cc22;
}
.failed {
    color: red;
}
#rightGreyText {
    color: #ccc;
    width: 100%;
    text-align: right;
    padding-top: 3px;
    font-style: italic;
}
.indented {
    padding-left: 10px;
}
label {
    cursor: pointer;
}
.input-checkbox {
    padding-top: 7px !important;
}
.input-checkbox input+.help-block {
    display: none;
}
.input-checkbox input:checked+.help-block {
    display: block;
    float: right;
    margin: 0;
}
.sup {
    vertical-align: sup !important;
}

.align-right {
    text-align: right;
}
.align-center {
    text-align: center;
}
.float-center {
    float: center;
}
.unselected,
.selected {
    display: inline-block;
}
.unselected {
    padding: 6px 10px 6px 10px;
    border: 1px solid #636363;
    margin-left: 8px;
    margin-right: 8px;
    color: #636363;
}
.selected {
    padding: 6px 10px 6px 10px;
    color: white;
    background-color: #636363;
    border: 1px solid #636363;
    margin-left: 8px;
    margin-right: 8px;
}
.bigger {
    font-size: 14px;
}
.padded {
    padding: 12px;
}
.bigger input {
    font-size: 16px;
}
.required-star {
    color: red;
}
.full-width {
    width: 100%;
}
.bigbutton {
    font-size: 18px !important;
}
.bold {
    font-weight: bold;
}
.correct {
    border: 2px solid #00cc22;
}
.incorrect {
    border: 2px solid red;
}
.hidden {
    display: none !important;
}
.text-input {
    width: 130px;
}
.text-input-wide {
    width: 230px;
}
.text-input-thin,
#server-hidden-settings input[type="number"] {
    width: 100px;
}
.input-group-bw {
    width: 150px;
}
.disabled-text {
    text-decoration: line-through;
    color: #ccc;
}
#serverQuote {
    opacity: 0.8;
    box-shadow: none !important;
    white-space: normal;
    width: 100%;
}
#host-tip {
    margin-bottom: 5px;
}
.tips {
    display: inline;
    font-style: italic;
    color: #888;
}
.error-text {
    display: inline;
    color: red;
}
#bandwidth {
    display: inline-block;
}

/***
    Bootstrap overwrites
***/

* {
    border-radius: 0 !important;
}
#content a, 
#content a:hover, 
#content a:active, 
#content a:visited {
    color: #555;
}
.btn {
    box-shadow: 1px 1px 1px rgba(0, 0, 0, .1) !important;
    min-width: 150px;
}
.btn .glyphicon,
a .glyphicon {
    top: 2px;
}
small {
    color: #777;
}