body {
	background-color: #fff;
	color: #000;
	font:normal 13px arial;
	line-height: 18px;
	padding: 0px;
	margin: 0px;
	text-align: center;


}
#hide{ display:none;}
#Container {
margin: 0 auto;
text-align: left;
width:90%;
color:black;
min-width:700px;
margin-top:3px;

}

#testing{
display:inline;
}
#Content{
	border-top: 1px dotted #ccc;


}
#Left {
width:100px;
height:100%;
float:left;
padding:0px;
margin:0px;
}
#Content{width:100%;margin:0;padding:0;}

#RightContainer {
/*height:600px;*/
margin-left: 100px;
padding:10px 0 10px 10px;
/*overflow:auto;*/

}
#Right{
    width:100%;
}
#Right_bottom{
    width:100%;
}
#Content{
clear:both;

}

#Footer{
clear:both;
border-top: 1px dotted #ccc;
padding:15px;
text-align: center;
/*line-height:30px;*/
}

#menu{
list-style-type: none;
padding:10px;
margin:0px;
position:relative;
}

.progressBar {
  background-color: #fff; 
  border: 1px solid #000; 
  height: 6px;
  width: 99%;
  margin: 0px;
  margin-left: auto; 
  margin-right: auto;
  padding: 0px;
  text-align: left;
}
.progressBartop {
  background-color: #fff; 
  border: 1px solid #ccc; 
  height: 14px;
  width: 99%;
  margin: 0px;
  margin-left: auto; 
  margin-right: auto;
  padding: 0px;
  text-align: left;
}
.percentageBar {
  position: relative; 
  background-color: #4B4545; 
  height: 100%;
}


h1{
background-color:#4B4545;
color:#fff;
padding:8px;
width:100px;
font-size:20px;
display:inline;
}
h2{
font-size:12px;
margin:0 0 0 8px;
}
h3{
display:inline;
line-height: 20px;
}
#title{
border:none;
color:#fff;
display:inline;

}
#rss{border:none;}

a:visited{
color:#000;
display:inline;
margin-top:-5px;
}
a {
color:black;
cursor:pointer;
text-decoration:underline;
}
a.config{
color:#4B4545;
}
a:hover{
color:red;
}

fieldset.EntryFieldSet {
  vertical-align: top;
  display: inline;
  padding: 7px;
  margin: 5px;
}
.secondul{
padding-left:10px;
margin:0px;
list-style-type: none;
}
.kbs{padding-left:2px;}

#divchart,#divpie{
margin: 0 auto;
}
.status{float:right;padding:0px;line-height: 22px;margin:0px 0px 10px 0px;}

.paused{background-color:red;padding:2px;}
.downloading{background-color:#00FF00;padding:2px;}
.idle{background-color:yellow;padding:2px;}
.spacer{color:white;margin-top:100px;margin-bottom:100px;}

#divpie2,#divpie3{display:inline;clear:none;}
#feed{display:inline;text-decoration: none;color:white;}
.bold{font-weight: bold; }


fieldset {
  margin-bottom: 10px;
  width:100%;
  border:none;
}
form.cmxform .label,form.cmxform input {
	display: block;
	width: 150px;
	float: left;
	margin-bottom: 15px;
}
input.schedulesubmit {
	display: inline;
	width: 30px;
	float: none;
	margin: 0;
    margin-left: 3px;;
}
.addnewselect{
	display: block;
	width: 90px;
    height:22px;
	float: left;
	margin-bottom: 15px;
}
.scriptselect{
	display: block;
	width: 100px;
    height:22px;
	float: left;
	margin-bottom: 15px;
}
form.cmxform .label {
	text-align: left;
	width: 150px;
	padding-right: 20px;
    padding-left:10px;
}

.clear{
	clear: left;
}
form.cmxform input.radio{
  width:15px;float:none;display:inline;margin-bottom: 0px;
}

.submitindent{
	margin-left: 183px;/*margin-bottom:-5px;*/}

.submitindent2{
	display: inline;
	float: none;
    height:22px;
    width:80px;
}

.select{width:154px}
.tips {
    margin-left:30px;
}

hr
{
margin-top:-6px;
margin-bottom:20px;
color:#ccc;
background-color:#ccc;
border:none;
height:1px;
/*width:500px;*/
}
#queueTable,#catTable, #rssTable, #historyTable{width:100%;}
th{background-color:#4B4545;color:white;}
tr{background-color:#f5f5f5;}
#versions{font-size:8pt;}
#downloadinfo{font-size:8pt;}
#help{float:right;}
.finished{background-color:#ffd9d9;}
.active{background-color:#deffd9;}
.waiting{background-color:#ffffd9;}

.delicon, .pauseicon, .resumeicon{
border:none;
color:#fff;
display:inline;
margin:0px;
padding:0px;
margin-bottom:-3px;
}
.imglink{
border:none;
}
#sortable_table{
width:99%;
}
.submitstatus{
display:inline;
padding-left:10px;
}
.centerLinks{
/*display:inline;
margin-left:20px;*/
text-align:center;
}


.server{
float:left;
width:360px;
padding-right:50px;
}
.float{float:left;}
#logo{float:left;}
/*#divchart{float:left;margin-left:50px;}*/

#warnings{
height:200px;
width:100%;
overflow:auto;}

.job_progress{position: relative;   background-color: #9bf;   height: 6px;}

.normalheight, .doubleheight, .tripleheight, .pattern, .result, .heading1, .heading2, .heading3 {float:left;}
.normalheight, .doubleheight, .tripleheight, .heading1 {width:125px;}
.normalheight {}
.doubleheight {height:30px;}
.tripleheight {height:60px;}
.row{clear:left;}
.pattern, .heading2 {width:70px; background-color:#f5f5f5;}
.result, .heading3  {width:125px;background-color:#f4f4f4;}
#Keytable {text-align:center; }
.heading1, .heading2, .heading3{font-weight:bold; background-color:black; color:white;}
.feedEnabled{color:green;}
.feedDisabled{color:red;}
#speedbox{width:36px;display:inline;}
.small{font-size:10px;}
#Keytable,#Keytable2,#Keytable3,.hidden{display:none;}
.actionTD{min-width:113px;}
.left-margin{padding-left:20px;}
.line-height{line-height:22px;}
td {padding:4px !important;}
.center{text-align:center;}
.expander {
    text-decoration:none; 
    margin:0px 2px 0px 2px;
    padding:0px 2px 0px 2px;
}
a.expander:hover {
    color:white;
    background-color:black;
}
.expander-minus {
    margin:0px 2px 0px 2px;
    padding:0px 4px 1px 4px;
}

.fail_message{color:red;}
.right_align{text-align:right;}
.row_hidden{visibility:collapse;}
a.columnLinks{color:white}
.expandColumn, .orderColumn{width:10px;}
.timeleftColumn, .sizeColumn{width:80px;}
.ageColumn, .editColumn, .actionsColumn, .linksColumn{width:50px;}
.statusColumn{width:100px;}
.searchBox{float:right; display:inline;}
.header-margin{padding-top:34px;}
.light-grey a{color:#999;}
.action_message{color:green;}
.fill-td{width:97%;}
a.unselected {
    padding:1px 5px 1px 5px;
    text-decoration:none;
}
span.unselected {
    padding:1px 5px 1px 5px;
}
.selected, a.unselected:hover {
    padding:1px 5px 1px 5px;
}

#hiddenNav {
    /*display:none;*/
}

.underline{text-decoration:underline;}
#hiddenTimedPause{display:none;}

.server-test {
    border: 1px dotted #ccc;
    padding:5px;
}

.success {
    color: green;
}

.failure {
    color: red;
}