body {
    overflow-y: scroll;
    background-color: #E4E4E4;
}
#logo {
    display: block;
    margin: auto;
    margin-top: 3px;
}

#content {
    color: #000;
    padding: 15px 20px 20px;
    font-size: 13px;
    padding-top: 65px;
    padding-bottom: 20px;
}
.colmask {
    z-index: 20;
    position: relative;
    clear: both;
    float: left;
    overflow: visible;
    border: 1px solid #dfdede;
    background-color: #FFF;
    border: 1px solid rgba(0, 0, 0, 0.2);
    width: 100%
}
.section, .Servers form, #addFeed, #addFeedContent {
    border-bottom: 1px solid #dfdede;
    overflow: auto;
}
.section:last-child, .Servers form:last-child {
    border: 0;
}
.col2 {
    z-index: 10;
    float: left;
    overflow: auto;
    padding: 12px;
    width: 250px;
}
.col1 {
    z-index: 11;
    width: 100%;
    float: none;
    padding: 12px 0 0;
}
.col2 h3 {
    background: none repeat scroll 0 0 #666;
    color: #fff;
    padding: 6px 8px;
    vertical-align: middle;
    margin: 0;
    font-size: 1.4em;
    overflow: hidden;
    white-space: nowrap;
    text-overflow: ellipsis;
    position: relative;
}
.col2 h3 a {
    display: none;
    position: absolute;
    right: 8px;
    bottom: 0.4em;
    color: white;
    font-size: 0.8em;
}
.section:hover .col2 h3 {
    padding-right: 25px;
}
.section:hover .col2 h3 a {
    display: inline-block;
}
.main-helplink {
    display: block;
    position: static;
    float: right;
    color: black !important;
    padding: 0px;
    font-size: 1.2em;
}
.example {
    background-color: #fefeee;
}
.presets input {
    margin: 2px 0;
}
label.config,
.field-pair h5 {
    overflow: auto;
    float: left;
    width: 24%;
    font-weight: 700;
    display: block;
    text-align: left;
    margin-right: 10px;
    margin-left: 20px;
    padding-top: 5px;
}
label.config-hover {
    cursor: pointer;
    
}
label.narrow {
    width: 200px!important;
}
label.wide,
.field-pair h5 {
    width: auto!important;
}
.field-pair h5 {
    margin-bottom: 5px;
}
.desc {
    display: block;
    margin: 0 0 0 24%;
    padding: 3px 0 0 31px;
    font-size: 12px;
    font-style: italic;
}
.desc.narrow {
    margin: 0 0 0 200px!important;
}
input[type="checkbox"]+.desc {
    padding-top: 6px;
} 
.desc-check {
    font-size: 12px;
    font-style: italic;
    padding: 0 1px;
}
.col2 p {
    font-size: 12px;
    color: #666;
    margin: 1em 0;
}
.field-pair {
    padding: 6px;
    clear: both;
    float: none;
    overflow: hidden;
    min-width: 555px;
}
.field-pair:nth-child(odd),
.Key tr:nth-child(odd),
.tab-pane tr:nth-child(odd),
.even {
    background-color: #F8F8F8;
}
.field-pair:last-child {
    background-color: transparent;
}
.alt,
.infoTableSeperator.alt {
    background-color: #F8F8F8;
}
.success {
    color: #5cb85c !important;
}
.typeahead.dropdown-menu {
    padding: 0;
    min-width: 300px;
    margin: 0;
}
.typeahead.dropdown-menu>li>a {
    padding-left: 12px;
    font-size: 13px;
}
.typeahead.dropdown-menu>.active>a, 
.typeahead.dropdown-menu>.active>a:focus, 
.typeahead.dropdown-menu>.active>a:hover {
    color: black;
    background-color: #E5E5E5;
}
#content fieldset {
    border: 0;
    outline: 0;
    min-width: 350px;
    margin: 0;
    padding: 0 12px 12px 6px;
}
textarea, select, input[type="date"], input[type="datetime"], input[type="datetime-local"], input[type="email"], input[type="month"], input[type="number"], input[type="password"], input[type="search"], input[type="tel"], input[type="text"], input[type="time"], input[type="url"], input[type="week"] {
    padding-left: 5px;
    border: 1px solid #d4d0c8;
}
select option {
    padding-left: 5px;
}
#check_new_rel option, #check_new_rel {
    text-transform: capitalize;
}
input[type='checkbox'] {
    display: block;
    white-space: nowrap;
    padding: 0;
    margin-left: 5px;
}
textarea:hover, input[type="date"]:hover, input[type="datetime"]:hover, input[type="datetime-local"]:hover, input[type="email"]:hover, input[type="month"]:hover, input[type="number"]:hover, input[type="password"]:hover, input[type="search"]:hover, input[type="tel"]:hover, input[type="text"]:hover, input[type="time"]:hover, input[type="url"]:hover, input[type="week"]:hover, textarea:focus, input[type="date"]:focus, input[type="datetime"]:focus, input[type="datetime-local"]:focus, input[type="email"]:focus, input[type="month"]:focus, input[type="number"]:focus, input[type="password"]:focus, input[type="search"]:focus, input[type="tel"]:focus, input[type="text"]:focus, input[type="time"]:focus, input[type="url"]:focus, input[type="week"]:focus {
    background-color: #fffff0;
    border: 1px solid #aaa;
}
.col1 input[type='checkbox'] {
    position: absolute;
    top: auto!important;
    height: 16px;
    width: 16px;
    margin-left: 0;
}
.col2 input[type='checkbox'] {
    height: 16px;
    width: 16px;
    margin-top: 0;
}
.readonly {
    background-color: #eff;
}
.multiple_cats {
    padding: 5px 10px!important;
    margin-top: 2px;
}
.padding {
    padding: 6px;
}
.disabled {
    color: #aaa;
}
.padTable p {
    margin-top: 0;
}
.nomargin {
    margin: 0;
}
.copyright {
    margin: 0;
    padding: 6px;
}
.path {
    color: #000;
    font-weight: 400;
}
.darkred {
    color: #d9534f;
}
.scheduleEntry {
    padding: 8px 0;
}
.time {
    padding: 0 10px;
    color: #000;
    font-weight: 700;
}
.frequency {
    padding-right: 5px;
    letter-spacing: .5px;
}
.padTable {
    padding: 15px;
    overflow: auto;
    clear: both;
}
.padTable h3 {
    margin-top: 0;
}
tr.separator {
    height: 30px;
}
.catTable th, .catTable td {
    padding: 5px;
}
.catTable th {
    text-align: center;
}

.RSS .rss-section input[type="text"] {
    max-width: 180px;
}

.RSS .catTable select {
    max-width: 120px;
}
.RSS input[type="checkbox"] {
    width:16px;
    height: 16px;
    margin-top: 0px;
}
.RSS form[action="add_rss_feed"] tr:nth-child(even) {
    border: 1px solid #E5E5E5;
}
.Config .table {
    margin: 0;
    border: 1px solid #ddd
}
.Config .colmask {
    margin-bottom: 12px;
}
.Config .padding {
    padding: 13px;
}
.Config th {
    width: 20%;
    min-width: 150px;
    padding-left: 15px !important;
}
.Config .label {
    font-size: 90%;
}
.Config td .glyphicon-question-sign  {
    color: black !important;
    top: 3px;
}
.tab-pane thead tr {
    background-color: transparent !important;
}
.default {
    background-color: #ffffe0;
    border-bottom: 1px solid #aaa;
    border-top: 1px solid #ddd;
}
.hidden {
    visibility: hidden;
    display: block!important;
}
.float-left {
    float: left;
}
.float-right {
    float: right;
}
.align-left {
    text-align: left;
}
.align-right {
    text-align: right;
}
.align-center {
    text-align: center;
}
.nowrap {
    white-space: nowrap;
}
.Key {
    display: none;
    border: 1px solid #ccc;
    width: 425px;
}
.Key th {
    background: none repeat scroll 0 0 #666;
    color: #fff;
    margin: 0;
    padding: 3px 9px;
}
.Key td {
    padding: 2px;
}
.Key tr {
    border-bottom: 1px solid #ccc;
}
#infoTable {
    font-size: 12px;
    width: 100%}
#infoTable tr {
    line-height: 17px;
}
.infoTableHeader, .infoTableCell {
    padding: 6px;
    word-break: break-all;
}
.infoTableHeader {
    font-weight: 700;
    width: 160px;
}
.infoTableSeperator {
    border-top: 1px solid #ddd;
}
.icon {
    padding-left: 5px;
}
.modal-content {
    box-shadow: none;
}
.modal-backdrop {
    background-color: white;
}
.modal-backdrop.in {
    opacity: 0.8;
}
.modal-body {
    padding-bottom: 0;
}
.modal-dialog {
    padding: 3px;
}
.modal-body h4 {
    word-break: break-all;
}
.modal-header {
    background-color: #3C3C3C;
    color: white;
    border: 0;
    padding: 10px 12px;
}
.modal-header .close {
    opacity: 0.7;
    background: transparent;
    color: white;
    text-shadow: none !important;
    font-weight: normal !important;
    font-size: 28px;
}
.modal-header .close:hover {
    opacity: 1;
}
.modal-dialog {
    transform: scale(0.7) !important;
    -webkit-transform: scale(0.7) !important;
    opacity: 0 !important;
    transition: all 0.3s !important;
}

.modal.in .modal-dialog {
    transform: scale(1) !important;
    -webkit-transform: scale(1) !important;
    opacity: 1 !important;
}
.fileBrowser {
    margin-left: -1px!important;
}
#filebrowser_modal .list-group-item {
    font-weight: bold;
}
#filebrowser_modal .list-group-item span {
    margin-right: 10px;
    top: 0;
}
#filebrowser_modal .checkbox {
    float: left;
    margin:  8px 5px 0x;
}
#filebrowser_modal .checkbox input {
    margin-top: 1px;
}
#filebrowser_modal .checkbox input+span {
    opacity: 0.6;
    font-weight: bold;
}
#filebrowser_modal .checkbox input:checked+span,
#filebrowser_modal .checkbox span:hover {
    opacity: 1;
}
/* Fix for shifting scrollbars when opening modals */
.modal-open[style="padding-right: 17px;"] .navbar-fixed-top {
    padding-right: 17px;
}
.modal-open[style="padding-right: 15px;"] .navbar-fixed-top {
    padding-right: 15px;
}
.modal-open[style="padding-right: 13px;"] .navbar-fixed-top {
    padding-right: 13px;
}
/* --  */
h2.activeRSS {
    margin-bottom: 10px;
}
.activeRSS a, .activeRSS a:visited {
    text-decoration: none;
    color: #000;
}
.activeRSS a:hover {
    color: #4b4742;
}
.activeRSS a:first-of-type {
    text-decoration: underline!important;
}
.favicon {
    background-position: center center!important;
    opacity: 1;
    top: -1px;
    height: 16px;
    width: 16px;
    float: left;
    margin-right: 2px;
}
.feed {
    text-decoration: none;
}
.feed_enabled {
    color: #000;
}
.feed_disabled {
    color: red !important;
}
#subscriptions {
    border: 1px solid #E5E5E5;
}
.data-row {
    border-top: 1px solid #E5E5E5;
}
#subscriptions td {
    border: 0 none;
    padding-top: .4em;
}
#subscriptions .chk {
    padding: 5px;
    padding-top: 8px;
    vertical-align: middle;
}
#subscriptions .title {
    font-weight: bold;
    padding-right: .3em;
    width: auto;
}
.ie6 .subscription-title {
    width: 20em;
}
.subscription-title,
.subscription-title:hover {
    width: 20em;
    overflow: hidden;
    white-space: nowrap;
    display: block;
    text-decoration: underline;
    color: black;
}
.controls {
    text-align: right;
    padding-right: 5px;
    white-space: nowrap;
}
.feed-row td {
    color: #777;
    max-width: 515px;
    padding: 0 0 .5em;
}
.feed-row div {
    padding-right: 10px;
    overflow:hidden;
    white-space: nowrap;
    text-overflow: ellipsis;
}
.tab-content .catTable {
    width: 100%;
}
.tab-content .catTable tbody {
    border: 1px solid #ddd;
}
.tab-content .catTable td {
    line-height: 2em;
}
.tab-content .catTable tr:hover td {
    background-color: #F2F2F2;
}
span.count {
    background-color: #6E6E6E;
    color: #fff;
    margin: -3px 0 0 5px;
    padding: 1px 10px;
}
a.current .count {
    background-color: #777!important;
}
a.current:hover .count {
    background-color: #333!important;
}
ul.tabs {
    margin: 0!important;
    padding: 0;
    height: 30px;
}
ul.tabs li {
    float: left;
    padding: 0;
    margin: 0;
    list-style-type: none;
}
ul.tabs a {
    float: left;
    font-size: 13px;
    display: block;
    padding: 5px 30px;
    text-decoration: none;
    border: 1px solid #dfdede;
    border-bottom: 0;
    color: black;
    margin-right: 2px;
    position: relative;
    top: 1px;
    outline: 0;
}
ul.tabs a:hover {
    background-color: #eee;
    color: #333;
}
ul.tabs li.active a {
    background-color: #eee;
    border-bottom: 1px solid #f8f4e7;
    color: #000;
    cursor: default;
}
.ui-tooltip-qrcode {
    max-width: 320px;
}
.ui-tooltip-qrcode img {
    margin: 0 auto;
}
.checkbox-days {
    float:  left;
}
.checkbox-days p {
    margin:  0 0 5px 0;
}
.checkbox-days input  {
    vertical-align: middle;
     margin-top:  -1px;
}

.checkbox-days label {
    padding: 2px 20px;
}
.rating-filter  {
    float:  left;
}
.rating-filter p  {
    margin:  0 0 10px 0;
}
.rating-filter select  {
    vertical-align: middle;
}
.rating-filter input  {
    vertical-align: middle;
     margin-top: -1px;
}
.rating-filter label  {
    display: inline-block;
     padding-left: 0px;
     width: 100px;
}
.rating-filter input[type="checkbox"]  {
    display: inline;
}
.rating-filter input[type="checkbox"] + label  {
    padding-left: 20px;
    padding-top: 5px;
     width: auto;
}
.rating-filter p > span:first-child  {
    float: left;
     width: 130px;
}
.rating-filter .desc  {
    display: block;
     margin: 0px;
     padding-left: 103px;
}

/** EDITS 2015 **/
* {
    border-radius: 0 !important;
}

.collapsing {
    -webkit-transition: none;
    transition: none;
}

input[type=checkbox], 
input[type=radio] {
    margin-top: 2px;
}

input[type="button"],
input[type="submit"] {
    color: #333;
    background-color: #fff;
    display:inline-block;
    padding:6px 12px;
    margin-bottom: 0;
    font-size:13px;
    font-weight:400;
    line-height:1.42857143;
    text-align:center;
    white-space:nowrap;
    vertical-align:middle;
    cursor:pointer;
    background-image:none;
    border:1px solid #ccc;
    height: 34px;
}

input[type="button"]:hover,
input[type="submit"]:hover {
    color: #333;
    background-color: #e6e6e6;
    border-color: #adadad;
}

input[type="text"],
input[type="email"],
input[type="url"],
input[type="number"],
input[type="password"],
textarea,
select {
    line-height:1.42857143;
    padding:6px 12px;
    border: 1px solid #d4d0c8;
    box-shadow: inset 0 1px 1px rgba(0,0,0,.075);
    display:inline-block;
    vertical-align:middle;
    max-width: 100%;  
    min-height: 34px;
    font-size: 13px;
    background-color: white;    
}

input[type="text"]:not([size]),
textarea {
    width: 300px;
}

input[type="number"],
input[type="text"].smaller_input {
    width: 120px;
}

.btn {
    font-size: 13px;
    height: 34px;
    box-shadow: 1px 1px 1px rgba(0,0,0,.1) !important;
}

.btn .glyphicon {
    top: 2px;
}

.btn .glyphicon-ok,
.btn .glyphicon-plus {
    top: 1px;
}

select {
    min-width: 200px;
    max-width: 300px;
}

input[disabled],
select[disabled] {
    background-color: #eee;
}

select[multiple] {
    width:100%;
    padding: 0 !important;
}

.col1 select[multiple] {
    max-width: 300px;
}

select[multiple] option {
    padding:6px 12px;
}

input[type="checkbox"] {
    margin-top: 5px;
}

.Scheduling select,
.Categories select,
.RSS select {
    min-width: 120px;
}

.Categories select {
    max-width: 150px;
}

.Categories input {
    max-width: 150px !important;
}

.Scheduling input[type="checkbox"] {
    margin-top: 0px;
    margin-left: -20px;
}

.navbar .container {
    padding-right: 0;
}

.navbar-default {
    background-color: #FFF;
    border-bottom: 1px solid rgba(0, 0, 0, 0.2);
    margin-bottom: 10px;
    
}

.navbar-fixed-top {
    z-index: 1000;
}

.navbar-default .navbar-brand {
    padding: 0 !important;
    margin: 0 !important;
    padding-right: 5px !important;
}
.navbar-default .navbar-brand-small {
    padding-right: 10px !important;
}
.navbar-default .navbar-brand-mobile {
    display: none;
}
.navbar-default .navbar-nav>li>a {
    color: black !important;
}

.navbar-default .navbar-nav>li>a:hover {
    background-color: #e2e2e2 !important;
}

.navbar-default .navbar-nav>li>a.active {
    background-color: #ECECEC !important;
}

.navbar-default .navbar-nav>li>a>span,
.navbar-default .navbar-nav>li>a>svg {
    display: none;
}

#navbar .glyphicon-folder-open {
    margin-right: 2px;
}

.rss-icon-svg {
    display: inline-block;
    margin-top: 1px;
    margin-bottom: -1px;
    width: 1em;
    height: 1em;
}
.rss-symbol { 
    stroke: none; 
    fill: white;
}

.col2 table {
    margin: 10px 0px;
}

.Servers .col2 .label {
    margin-top: 8px;
    font-size: 0.85em;
    float: right;
    line-height: 1.1em;
}

.Servers .col2.server-disabled .label {
    color: ##777 !important;
}

.Servers .col2 .label:nth-child(2) {
    opacity: 0.7;
}

.Servers .col2 label,
.Email .col2 label {
    margin: 0;
    margin-left: 4px;
    margin-top: 2px;
    cursor: pointer;
}

.Servers .col2 button:first-of-type {
    margin-bottom: 8px;
}

.server-disabled {
    background-color: #eee;
}

.Special .glyphicon-asterisk {
    margin-left: 8px;
    top: 3px;
    color: #666;
}

.fileBrowser .glyphicon {
    margin-right: 2px;
    top: 1px;
}

.main-restarting.in {
    text-align: center;
    opacity: 0.9;
    color: black;
    z-index: 2000;
    height: 100%;
}

.main-restarting.in div {
    position: relative;
    top: 30%;
    transform: translateY(-50%);
}

.main-restarting {
    display: block;
    background-color: white;
}

.main-restarting.in strong {
    font-size: 5em;
    display: block;
}

.main-restarting.in span {
    top: 10px;
}

.main-restarting small {
    font-size: 3rem !important;
}

.dotOne, .dotTwo, .dotThree {
    opacity: 0;
    animation: dot 1.3s infinite;
    animation-delay: 0.0s;
}

.dotTwo {
    animation-delay: 0.2s;
}

.dotThree {
    animation-delay: 0.3s;
}

@keyframes dot {
    0% { opacity: 0; }
    50% { opacity: 0; }
    100% { opacity: 1; }
}

@media screen and (min-width: 1200px) { 
    .Categories input[name="dir"] {
        max-width: 240px !important;
    }
}

@media screen and (min-width: 768px) and (max-width: 1200px) { 
    .navbar-default .navbar-nav>li>a>span,
    .navbar-default .navbar-nav>li>a:hover>strong,
    .navbar-default .navbar-nav>li>a.active>strong {
        display: inline;   
    }
    
    .navbar-default .navbar-nav>li>a>svg {
        display: inline-block;
    }
    
    .navbar-default .navbar-nav>li>a>strong,
    .navbar-default .navbar-nav:hover>li>a.active>strong {
        display: none;
    }
        
    .RSS #content,
    .Categories #content {
        width: 100%;
    }
}

@media screen and (max-width: 1000px) {
    .col1 {
        float: right;
        padding: 5px 0px;
    }
    
    .col1 h5 {
        margin-left: 20px;
    }
    
    .col2 {
        width: 100%;
        padding: 10px;
    }
    
    .col2 p {
        margin-left:  20px;
    }
        
    .Servers .col2 button:first-of-type {
        margin-bottom: 0;
    }
}

@media screen and (max-width: 768px) {
    .navbar-default .navbar-brand-small {
        display: none;
    }
    .navbar .container {
        padding-right: 15px;
    }
    .navbar-default .navbar-brand-mobile {
        display: block;
    }
    .navbar-collapse.in .navbar-nav>li>a>span,
    .navbar-collapse.collapsing .navbar-nav>li>a>span {
        display: inline-block;  
        width: 1.5em; 
    }
    
    .navbar-collapse.in .navbar-nav>li>a>svg,
    .navbar-collapse.collapsing .navbar-nav>li>a>svg {
        display: inline-block;
        margin-right: 0.5em;
    }
    
    .section .col2 h3 {
        padding-right: 25px;
    }
    
    .section .col2 h3 a {
        display: inline-block;
    }
    
    .main-restarting.in strong {
        font-size: 2em;
        font-weight: bold;
    }
    
    .main-restarting.in span {
        top: 2px;
    }
    
    .main-restarting.in small {
        font-size: 2rem !important;
    }

    #content {
        margin: 0 !important;
        padding: 0 !important;
        padding-top: 55px !important;
    }
    
    #content fieldset {
        min-width: 0;
        padding: 0 !important;
    }
    
    .colmask {
        border: none !important;
    }
    
    .col2 {
        padding: 10px;
    }
    
    .col2 h3 {
        margin: -10px -10px 0px;
        padding-left: 10px;
    }
    
    .col2 p {
        margin-left:  0px;
        margin-bottom: 0;
    }
    
    .col1 h5 {
        margin-left:  0px;
    }        
    
    label.config {
        margin-left: 0px;
        margin-bottom: 2px;
        width: 100%;
        float: none;
    }
    
    .desc {
        margin: 0;
        margin-left: 3px;
        margin-top: 2px;
        padding: 0 !important;
    }
    
    .col1 input[type="checkbox"] {
        position: inherit;
        float: left;
        margin-right: 5px;
        margin-top: 0px;
    }
    
    div.field-pair{
        min-width: 0;
        padding: 5px 10px;
    }

    .fileBrowserField {
        max-width: 65% !important;
    } 
    
    ul.tabs a {
        padding-left: 5px;
        padding-right: 5px;
    }
}

