/* --------------------- RESET ----------------*/
html, body, div, span, applet, object, iframe, h1, h2, h3, h4, h5, h6, p, blockquote, pre, a, abbr, acronym, address, big, cite, code, del, dfn, em, font, img, ins, kbd, q, s, samp, small, strike, strong, sub, sup, tt, var, dl, dt, dd, ol, ul, li, fieldset, form, label, legend, table, caption, tbody, tfoot, thead, tr, th, td {
  margin: 0;
  padding: 0;
  border: 0;
  outline: 0;
  font-weight: inherit;
  font-style: inherit;
  font-size: 100%;
  font-family: inherit;
  vertical-align: baseline;
}
/* remember to define focus styles! */
:focus {
  outline: 0;
}
body {
  line-height: 1;
  color: #000;
  background: #fff;
}
ol, ul {
  list-style: none;
}

/* tables still need 'cellspacing="0"' in the markup */
table {
  border-collapse: separate;
  border-spacing: 0;
}
caption, th, td {
  text-align: left;
  font-weight: normal;
}
blockquote:before, blockquote:after,
q:before, q:after {
  content: "";
}
blockquote, q {
  quotes: "" "";
}

/* ---------------- TYPOGRAPHY -----------------*/
body, input, textarea{
  font:75% 'Helvetica Neue', Arial, Helvetica, sans-serif;
  color:#000;
}
html>body{ font-size:12px; }
h1, h2, h3, h4{ font-family:'Century Gothic', 'AppleGothic', sans-serif; }
h1{ font-size:1.5em; }
h2{ font-size:1.25em; }
h3{ font-size:1em; }
p, ul, ul li, td{ font-size:1em; }
p,li,dt,dd,td,th{ line-height:1.5; }
p,dt,dd,td,th,ul,ol,dl{ margin-bottom:10px; }
strong { font-weight:bold; }
small { font-size:0.8333em; }
input, textarea{ font-size:1em; }

a:link,a:visited{
  color:#599FCF;
  text-decoration:none;
}
a:hover,a:active{
  text-decoration:underline;
}

.box_fatbottom,
.queueTable .eta,
.currentDownload,
.totalDownload {
  font-size:0.8333em;
}

/* -------------------------- PLUSH DEFAULT] -----------------*/
body {
  background: #B1B1B1;
  padding-bottom: 10px;
}

#page-wrap select {
  background-color: #E0E0E0;
  border: 0px;
  color: #000;
}

.new_release_link {
  color: #006400;
}

.pointer {
  cursor: pointer;
}

.clear {
  clear: both;
}

.float-right {
  float:right;
}

.nowrap {
  white-space: nowrap;
}

#warning_message {
  color: #8B0000;
  font-weight: bold;
}

#new_release {
  color: green;
  font-weight: bold;
}

#livetip {
  position: absolute;
  background-color: #cfc;
  padding: 4px;
  border: 2px solid #9c9;
  border-radius: 4px;
  z-index:99999;
  -webkit-border-radius: 4px;
  -moz-border-radius: 4px;
}


/* ------------- jQuery Pagination Plugin -------------- */

.pagination-perpage {
  float: left;
  margin-top: 2px;
}

.pagination {
  float: left;
  margin-left:10px;
}

.pagination a {
  background: #E0E0E0;
  border: solid 1px #B1B1B1;
  color: #000;
  cursor: pointer;
  text-decoration: none;
}

.pagination a:hover { border: solid 1px #000; }

.pagination a, .pagination span {
  display: block;
  float: left;
  margin-right: 5px;
  padding:3px 5px;
}

.pagination .current {
  background: #FFF;
  border: solid 1px #000;
  color: #000;
}

.pagination .current.prev, .pagination .current.next {
  background: #E0E0E0;
  border-color: #B1B1B1;
  color: #B1B1B1;
}

.pagination .loading { background: #FFF url('images/loading-spinner.gif') 50% 50% no-repeat; }


#add_nzb_modal select {
  width:190px;
}

#add_nzb_pp td{
  padding: 5px;
}

#add_nzb_hr {
  margin-top: 15px;
  margin-bottom: 15px;
}

#add_nzb_hr hr {
  height: 1px;
  border: 0;
  color: #B1B1B1;
  background-color: #B1B1B1;
  width: 95%;
  text-align: left;
  margin: 0 auto 0 0;
}

/*---------------- navigation -------------- */

.wrapper {
  width:100%;
  margin:auto;
}
.navigation {
  width: 100%;
  height: 30px;
  position: relative;
  padding: 0;
}
.navigation ul {
  margin: 0;
  padding: 0;
}

.navigation ul.menu {
  height: 30px;
}
.navigation ul#navigation_menu_left {
  margin-left:17px;
  float:left;
}
.navigation ul#navigation_menu_right {
  margin-right:17px;
  float:right;
}

.navigation ul li.active a{
  background-color:#000;
  background: -webkit-gradient(linear, right top, right bottom, from(#000), to(#333) );
  background: -moz-linear-gradient(top, #000 0%, #333 100%);
}

.navigation ul.menu li {
  list-style: none;
  float:left;
  height: 25px;
  text-align: center;
  background-color:#000;
  background: -webkit-gradient(linear, right top, right bottom, from(#000), to(#333) );
  background: -moz-linear-gradient(top, #000 0%, #333 100%);
  }

.navigation ul li a {
  color:#efefef;
  font-weight:bold;
  display: block;
  padding: 0 8px 0 0;
  border-left: 1px solid rgba(255,255,255,0.1);
  border-right: 1px solid rgba(0,0,0,0.1);
  text-align: center;
  line-height: 25px;
  background-color:#363636;
/*    background : -webkit-gradient(linear, right top, right bottom, from(rgb(168,168,168)), to(rgb(69,69,69)));
  background : -moz-linear-gradient(top, rgb(168,168,168), rgb(69,69,69));
  -webkit-transition-property: background;
  -webkit-transition-duration: 700ms;
*/
  -moz-transition-property: background;
  -moz-transition-duration: 700ms;
}

.navigation ul.menu li.first, .navigation ul li.first a {
  -moz-border-radius-bottomleft: 10px;
  -webkit-border-bottom-left-radius: 10px;
}

.navigation ul.menu li.last, .navigation ul li.last a {
  -moz-border-radius-bottomright: 10px;
  -webkit-border-bottom-right-radius: 10px;
}

.navigation ul li a:hover {
  background: transparent none;
  text-decoration:none;
}

.navigation ul li a img {
  opacity: .25;
  margin:0 8px -3px 6px;
}

.navigation ul li.active a img {
  opacity: 1.00;
}

.navigation ul li a:hover img {
  opacity: 1.00;
}

.navigation ul li a span {
  padding-left:26px;
  background-repeat:no-repeat;
  background-position:6px 0px;
}

.navigation ul li a span.icon_nav_download { background-image: url('images/nav/downloads-grey.png'); }
.navigation ul li.active a span.icon_nav_download, .navigation ul li a:hover span.icon_nav_download { background-image: url('images/nav/downloads.png'); }
.navigation ul li a span.icon_nav_connections { background-image: url('images/nav/connections-grey.png'); }
.navigation ul li.active a span.icon_nav_connections, .navigation ul li a:hover span.icon_nav_connections { background-image: url('images/nav/connections.png'); }
.navigation ul li a span.icon_nav_rss { background-image: url('images/nav/rss-grey.png'); }
.navigation ul li.active a span.icon_nav_rss, .navigation ul li a:hover span.icon_nav_rss { background-image: url('images/nav/rss.png'); }
.navigation ul li a span.icon_nav_config { background-image: url('images/nav/config.png'); }
.navigation ul li.active a span.icon_nav_config, .navigation ul li a:hover span.icon_nav_config { background-image: url('images/nav/config.png'); }

.navigation ul li a span.icon_nav_addnzb { background-image: url('images/nav/addnzb-grey.png'); }
.navigation ul li.active a span.icon_nav_addnzb, .navigation ul li a:hover span.icon_nav_addnzb { background-image: url('images/nav/addnzb.png'); }
.navigation ul li a span.icon_nav_options { background-image: url('images/nav/options.png'); }
.navigation ul li.active a span.icon_nav_options, .navigation ul li a:hover span.icon_nav_options { background-image: url('images/nav/options.png'); }
.navigation ul li a span.icon_nav_refresh { background-image: url('images/nav/refresh-grey.png'); }
.navigation ul li.active a span.icon_nav_refresh, .navigation ul li a:hover span.icon_nav_refresh { background-image: url('images/nav/refresh.png'); }
.navigation ul li a span.icon_nav_help { background-image: url('images/nav/help-grey.png'); }
.navigation ul li.active a span.icon_nav_help, .navigation ul li a:hover span.icon_nav_help { background-image: url('images/nav/help.png'); }

#nav_text_right {
  clear:both;
  float:right;
  padding: 0 20px 0 20px;
  font-size:90%;
}
#nav_text_left {
  clear:left;
  padding: 0 20px 0 25px;
  font-size:90%;
}
#nav_text_left a, #nav_text_right a {
  color:#000;
}
#last_warning {
  color:#000;
}

/* ------------------------- queue + history overall container ----------- */

/* sprite-main.png */
#main_menu ul li a span.sf-sub-indicator { background-position: 0 5px; }
#main_menu ul li ul li a span.sf-sub-indicator { background-position: 0 -16px; }

.main_sprite_container { background: url('images/sprite-main.png') no-repeat top right; }

.sprite_arrow_down{ background-position: 0 0;  }
.sprite_arrow_right{ background-position: 0 -19px;  }
.sprite_btm_corner_left{ background-position: 0 -38px;  }
.sprite_btm_corner_right{ background-position: 0 -74px;  }
.sprite_h_purge{ background-position: 0 -110px;  }
.sprite_h_search{ background-position: 124px -141px;  } /* 124px */
.sprite_hv_download{ background-position: 0 -167px;  }
.sprite_hv_error{ background-position: 0 -193px;  }
.sprite_hv_filejoin{ background-position: 0 -219px;  }
.sprite_hv_unpack{ background-position: 0 -245px;  }
.sprite_hv_report{ background-position: 0 -271px;  }
.sprite_hv_script{ background-position: 0 -297px;  }
.sprite_hv_star{ background-position: 0 -323px;  }
.sprite_hv_stats{ background-position: 0 -349px;  }
.sprite_hv_repair{ background-position: 0 -375px;  }
.sprite_progress_done{ background-position: 0 -401px;  }  /* queue.tmpl */
.sprite_progressbar_bg{ background-position: 0 -430px;  }
.sprite_q_menu_pausefor{ background-position: 0 -459px;  }
.sprite_q_menu_pausefor_on{ background-position: 0 -521px;  }
.sprite_q_pause{ background-position: 0 -552px;  }
.sprite_q_pause_on{ background-position: 0 -583px;  }
.sprite_q_queue{ background-position: 0 -614px;  }
.sprite_q_queuesfHover{ background-position: 0 -645px;  }
.sprite_q_queue_on{ background-position: 0 -676px;  }
.sprite_q_menu_pauseforsfHover{ background-position: 0 -490px;  }
.sprite_q_stats_complete{ background-position: 0 -707px;  }
.sprite_q_stats_incomplete{ background-position: 0 -733px;  }
.sprite_q_stats_needspace{ background-position: 0 -759px;  }
.sprite_ql_cross{ background-position: 0 -785px;  }
.sprite_ql_cross_on{ background-position: 0 -811px;  }
.sprite_ql_grip_pause{ background-position: 0 -837px;  }
.sprite_ql_grip_pause_on{ background-position: 0 -861px;  }
.sprite_ql_grip_resume{ background-position: 0 -885px;  }
.sprite_ql_grip_resume_on{ background-position: 0 -909px;  }
.sprite_rounded_area_bg{ background-position: 0 -933px;  }
.sprite_rounded_area_bg_edged{ background-position: 0 -965px;  }
.sprite_sabnzbdplus_logo{ background-position: 0 -997px;  }
.sprite_sabnzbdplus_plush{ background-position: 0 -1047px;  }
.sprite_tile_qh_seperator{ background-position: 0 -1070px;  }
.sprite_topleft_queue{ background-position: 0 -1084px;  }
.sprite_topright_queue{ background-position: right -1161px;  }  /* right */

/* tile-x sprite */
.tilex_sprite_container { background: url('images/sprite-tilex.png') repeat-x top right; }
.sprite_tile_bottom_wrap { background-position: 0 0; }
.sprite_tile_qh { background-position: 0 -27px; }
.sprite_tile_qhistory { background-position: 0 -43px; } /* subtract 16 from above */
.sprite_tile_qh_bottom { background-position: 0 -95px; }
.sprite_tile_znavbar { background-position: 0 -134px; }

/* tile-y sprite */
.tiley_sprite_container { background: url('images/sprite-tiley.png') repeat-y top right; }
.sprite_tile_page_wrap { background-position: 0 0; }
.sprite_tile_queue_wrap { background-position: bottom right; }

.box_fatbottom {
  color: #606060;
  height: 37px;
  text-align: center;
}

.left_stats, .right_stats {
  cursor: default;
  padding: 12px 16px 0 20px;
}

#box_fatbottom_queue .left_stats {
  padding-top:6px;
}

.plush_wrapper {
  display:block;
  height:15px;
  margin:0 auto;
  padding:13px 0 0;
  text-indent:-5000px;
  width:58px;
}
.plush_wrapper .main_sprite_container { height:15px; }

#history_stats { padding-top: 16px; }

.box_fatbottom .left_stats { float: left; }

.box_fatbottom .right_stats { float: right; }

.box_fatbottom strong { color: #000; }

.box_banner_menu {
  display: inline;
  float: right;
  list-style: none;
  margin: 10px 0;
  padding: 0;
}

.box_banner_menu li { float: right; }

/* keep .h_menu_verbose and .h_menu_purge up with the above (buttons top-right of history) */
.sprite_h_purge {
  cursor: pointer;
  display: none;
  margin: 6px;
  padding-bottom: 24px;
  padding-right: 28px;
}

#page-wrap {
  clear:both;
  margin-left: 0px;
  margin-right: 0px;
}

#master-width {
  clear:both;
  margin: 0px auto;
  width: 90%;
}

.box_banner_hr {
  background-repeat: repeat-x;
  height: 4px;
}

.box_banner_history {
  clear:both;
  height:51px;
  overflow:hidden;
}

.box_banner_history h1 {
  color: #c7c7c7;
  cursor: default;
  float: left;
  font-weight: normal;
  padding:17px 8em 11px 21px;
}

.box_banner_r {
  float:right;
  height: 52px;
  margin-left: 1px;
  width:350px;
}

.listSearchBox {
  color: black;
  width: 120px;
  border: 0px;
  margin: 7px 9px 1px 1px;
  padding-right:20px;
  border: 1px solid #444;
  background-color: #e5e5e5
}
.listSearchBox:hover,.listSearchBox:focus {
  border: 1px solid gray;
}

#queueSearchForm {
  display: block;
  height: 0px;
  text-align: right;
  margin-right: 151px;
}
#queueSearchBox {
  width: 121px;
  margin-top: 35px;
}
/* a window narrower than this doesn't have room for the search box, so hide it */
@media screen and (max-width:900px) { #queueSearchForm { display: none; } }

/* a window wider than this has enough room for a bigger box, so grow it to line up with histsearchbox below it */
@media screen and (min-width:1190px) { #queueSearchBox { width: 121px; background-position-x: 125px;} }

#hdr-queue {
  height: 67px;
}

#hdr-queue .logo {
  float: left;
  height: 67px;
}

#hdr-queue .logo h1 {
  color: #c7c7c7;
  cursor: default;
  font-weight: normal;
  margin: 32px 0 0 38px;
  width:140px;
  overflow:hidden;
}

.queue-buttons {
  float:right;
  height:37px;
  padding-top:34px;
  padding-right:5px;
  position:relative;
  white-space:nowrap;
}

#speed-wrapper {
  margin-top:21px;
  float:none;
  padding:0;
}

#speed-wrapper a:hover {
  text-decoration:none;
}

.queue-buttons ul {
  float:left;
  margin-right:24px;
}

.queue-buttons #pause_int {
  color:#ccc;
  font-size:small;
  padding-right:16px;
  position:absolute;
  right:104px;
  text-indent:0;
  top:34px;
}
.queue-buttons ul li {
  float: left;
  display:block;
  height:21px;
  position:relative;
  text-indent:-5000px;
  width:21px;
}
.queue-buttons ul li.sprite_q_queue {
  margin-left:10px;
}

/* Queue right-side pull down menu: multiops/eoq/purge/sort */
.queue-buttons ul li ul{ /* Second Level */
  border-bottom:2px solid #403f3f;
  border-right:2px solid #403f3f;
  display:none;
  float:left;
  height:auto;
  right:1px;
  margin:0;
  position:absolute;
  top:21px;
  z-index:100;
  color:#aaa;
}

/* These are to make the menu width language sensitive,
  not usable because sort-table width isn't language sensitive

  .queue-buttons.lang-nl ul li ul{
    width:210px;
  }
  .queue-buttons.lang-de ul li ul,
  .queue-buttons.lang-fr ul li ul{
    width:220px;
  }
*/

.queue-buttons ul li.sprite_q_queue ul{
  right:0;
}
.queue-buttons ul li ul li{
  background:#1d1c1c;
  border-top:1px solid #474646;
  clear:left;
  height:auto;
  margin:0;
  text-indent:0;
  width:100%;
  padding-left: 5px;
}

.queue-buttons ul li ul li:first-child{
  border:0;
}
#speed-wrapper ul li ul li {
    padding: 5px;
}
#speed-wrapper ul li ul li:last-child {
    text-align: center;
}
#speed-wrapper ul li ul li:first-child input {
    text-align: center;
    vertical-align: bottom;
}
.queue-buttons ul li ul li a,
.queue-buttons ul li ul li span{
  color:#aaa;
  clear:left;
  float:left;
  margin:0;
  padding:4px 6px;
  text-align:left;
  width:188px;
}
.queue-buttons ul li ul li span {
    cursor: auto;
}
.queue-buttons ul li ul li a:hover{
  color:#eee;
}
.queue-buttons ul li.sprite_q_queue ul li a:hover {
  text-decoration:none;
}
.queue-buttons ul li ul li #onQueueFinish-option{
  width:130px;
}
.queue-buttons ul li ul li ul,
  .queue-buttons ul li.sprite_q_queue ul li ul{   /* Third Level */
  border:0;
  position:absolute;
  right:200px;
  top:0;
}
/*
  .queue-buttons.lang-nl ul li ul li ul,
    .queue-buttons.lang-nl ul li.sprite_q_queue ul li ul{
    right:245px;
  }
  .queue-buttons.lang-de ul li ul li ul,
    .queue-buttons.lang-de ul li.sprite_q_queue ul li ul,
  .queue-buttons.lang-fr ul li ul li ul,
    .queue-buttons.lang-fr ul li.sprite_q_queue ul li ul{
    right:265px;
  }
*/
#queue-wrap {
  padding:0 16px;
}

#btm-wrapper {
  clear: both;
  height: 26px;
}
#btm-wrapper div {
  background-repeat: no-repeat;
  float: left;
  height:26px;
  width:26px;
}

#btm-wrapper .sprite_btm_corner_right {
  float: right;
}

#queue_sort_list {
  margin-right:-202px;
  margin-top:26px;
  width:221px;
  z-index:999999;
  border:1px solid #403f3f;
  border-left:0px;
}

/* ---------------- box header, above queue (sabnzbd logo & stats) ----------------- */
.centerpiece {
  margin: auto;
  padding:13px 0 0 20px;
  width:350px;
}

.sprite_sabnzbdplus_logo {
  margin: 10px 0 0;
}

.sabnzbd_logo {
  float: left;
  height: 40px;
  padding-left: 50px;
  padding-right: 50px;
}

#speed, #time-left {
  cursor: default;
  width: 68px;
  height: 12px;
  margin-top: 20px;
  padding: 3px 0 7px;
  text-align: center;
}

/* stats next to logo */
#speed {
  float: left;
  margin-left: 24px;
}

#time-left {
  float: left;
  margin-right: 24px;
  width: 88px;
}

#speed p, #time-left p {
  color: #7c7c7c;
  font-size: 9px;
}

#speed span, #time-left span {
  color: #dadada;
  font-size: 11px;
  font-weight: bold;
}

.show { display: inline; }

/*  queueTable shared by queue and history */
.queueTable {
  background-color: #e0e0e0;
  border-bottom: 1px solid #fff;
  border-collapse: separate;
  border-spacing: 0px;
  color: #606060;
  width: 100%;
}

.queueTable tr {
  background-color: #e0e0e0;
  cursor: default;
}
.queueTable tr:hover,
.queueTable tr.alt:hover {
  background-color: #f9f9f9;
  color: #000;
}
.queueTable tr select {
  background: #e5e5e5;
  border:1px solid #e0e0e0;
  color:#606060;
  padding:2px;
}

.queueTable tr select.hovering {
  background:#fff;
  border: 1px solid #c0c0c0;
  color: #000;
}

.queueTable tr.alt {
  background-color:#d0d0d0;
}
.queueTable tr.alt select {
  border:1px solid #d0d0d0;
  background: #d5d5d5;
}
.queueTable tr.alt select.hovering {
  border:1px solid #b0b0b0;
  background: #fff;
}

#queueTable tr:hover { cursor: move; }

.queueTable td {
  border-bottom: 1px solid #B1B1B1;
  border-top: 1px solid #FFF;
  padding: 3px 0;
  vertical-align: middle;
}

tr:hover .nzb_status_grippie {
  background-color: #f9f9f9; /* IE7 */
}

.queueTable td.download-title {
  width: 50%;
  height:28px;
}

.queueTable td.historyTitle {
  padding:0px;
  width: 50%;
  height:28px;
}

.nzb_status_col {
  width: 28px;
}

.nzb_status {
  display: inline;
  margin-right: 4px;
  padding-bottom: 2px;
  padding-left: 16px;
}

.nzb_status.Loaded { background: url('images/loading-spinner.gif') no-repeat; }

#queueTable .nzb_status {
  cursor: pointer;
}

.loading { /* remove? */
  background: url('images/loading-spinner.gif') 4px 50% no-repeat;
  display: inline;
  padding-left: 16px;
}
.initial-loading {
  background: url('images/loading-bar.gif') 4px 50% no-repeat;
  position:absolute;
  width: 138px;
  margin: 4px 0px 0px 100px;
}

.queueTable tr:hover td.historyTitle {
  background-color: #f9f9f9; /* IE7 */
  color: #000; }

.queueTable a {
  color: #404040;;
  text-decoration: none;
}

.queueTable tr:hover a {
  color: #000;
  border-bottom: 1px dotted #606060;
}

.queueTable tr:hover a:hover {
  border-bottom: 1px solid #606060;
  cursor: pointer;
}

.queueTable .eta {
  padding: 0 6px 0 6px;
  text-align: center;
}

.icon_nzb_remove {
  display: inline;
  padding-bottom: 2px;
  padding-left: 16px;
}
.icon_nzb_remove:hover {
  outline:none;
}

.sprite_ql_cross_on { cursor: pointer; }

#topmenu_bar {}

#multiops_bar {
  height:51px;
  color:grey;
}
#multiops_title {
  color: #C0C0C0;
  font-style: italic;
}
#multiops_inputs { }
#multiops_inputs .left {
  float:left;
  padding-left: 20px;
  padding-top:5px;
}
#multiops_inputs .right{
  float:right;
  padding-top:5px;
}
#multiops_inputs select {
  background: #e5e5e5;
  color:#606060;
  padding:1px;
  margin:0px;
  margin-right:3px;
}
#multiops_inputs select:hover {
  background:#fff;
  color: #000;
}
#multiops_inputs input {
  color:#000;
  border:0;
  margin-right:2px;
}
#multiops_inputs a {
  cursor:pointer;
  color:#FFF;
}

/* ------------------------------- Queue ------------------------------- */
/* progress bar & MB/left stats */
.sprite_progressbar_bg {
  width: 121px;
  padding-left:1px;
}

.sprite_progress_done {
  height: 19px;
  width: 121px;
}

.currentDownload, .totalDownload {
  color: #000;
  font-weight: bold;
  margin-top: 3px;
  z-index: 1;
}

.currentDownload small, .totalDownload small { font-weight: normal; }

.currentDownload {
  float: left;
  margin-left: 5px;
}

.totalDownload {
  float: right;
  margin-right: 5px;
}

.options {
  padding-right: 8px;
  text-align: right;
}

/* below queue, left side, limit */
#queue_view_preference, #history_view_preference { width: 120px; }

#queue_view_preference { margin-top: -5px; }

#stats_noofslots { display: none; }

/* below queue, right side, disk space */
.queue_footer_icon {
  display: inline;
  margin: 16px 4px 0px 20px;
  padding: 4px;
}

/* ------------------------------- History ------------------------------- */
.history_added {
  cursor: default;
  font-size: 75%;
}

tr:hover .history_added { color: black; }

.hvMessageFail {
  clear: both;
  color: #DE2424;
  font-size: 80%;
  font-weight: bold;
  width: 100%;
}

.hvMessageAction {
  clear: both;
  color: #00008B;
  font-size: 80%;
  font-weight: bold;
  width: 100%;
}

.hvMessageScript {
  clear: both;
  color: #006400;
  font-size: 80%;
  font-weight: bold;
  width: 100%;
}

.icon_history_verbose {
  cursor: help;
  display: inline;
  float: left;
  margin-right: 2px;
  width: 16px;
}
.hvFaded {
  opacity: .25;
  /*-ms-filter:"progid:DXImageTransform.Microsoft.Alpha(Opacity=25)";
  filter: alpha(opacity=25);*/
}

.pointer { cursor: pointer; }

/* ------------------------------- Ratings ------------------------------- */
.rating_stars_block_r {
  text-align: right;
}

.rating_stars_block_c {
  text-align: center;
}

.rating_vote_block {
  float: left;
}

.rating_stars {
  display: inline-block;
  margin-right: 10px;
}

.rating_flag {
  margin: 4px 10px 0px 85px;
}

.rating_flag_radio {
  margin: 5px 15px 5px 5px;
}

.rating_modal_extra {
  float: right;
  width: 330px
}

.rating_modal_expired {
  float: right;
}

.rating_modal_noopt {
  color: red;
  margin-left: 10px;
}
 
.rating_icon_vision {
  width: 16px;
  height: 16px;
  display: inline-block;
  margin-right: 5px;
  background: url('images/vision16.png') no-repeat top center;
}

.rating_icon_sound {
  width: 16px;
  height: 16px;
  display: inline-block;
  margin-right: 5px;
  background: url('images/sound16.png') no-repeat top center;
}

.rating_icon_thumbup {
  width: 20px;
  height: 20px;
  display: inline-block;
  background: url('images/thumbup20.png') no-repeat top center;
}

.rating_icon_thumbdown {
  width: 20px;
  height: 20px;
  display: inline-block;
  background: url('images/thumbdown20.png') no-repeat top center;
}

/* ------------------ MAIN MENU - courtesy jquery superfish plugin w/mods by jbw ---------------- */

#main_menu {
  padding-top:10px;
  text-align:center;
}

#main_menu ul {
  display:inline-block;
  list-style:none;
  margin:0 auto;
  padding:0;
  white-space:nowrap;
}
#main_menu ul li {
  float:left;
    position:relative;
}
#main_menu ul li a {
  display:block;
  float:left;
  margin:0 10px;
  padding:0 20px 0 5px;
  cursor:pointer;
  position:relative;
  text-align:left;
  border-bottom:1px;
}
/* refresh button below queue */
#manual_refresh_wrapper b { font-weight:normal; }
#manual_refresh_wrapper.refreshing b { border-bottom: 1px dotted white; }
#manual_refresh_wrapper.refresh_skipped b { border-bottom: 1px solid red; }

#main_menu ul li a span {
  background: url('images/sprite-main.png') no-repeat center left;
  display:block;
  height:1.5em;
  right:5px;
  position:absolute;
  text-indent:-5000px;
  top:0;
  width:10px;
}
#main_menu ul li ul{ /* Second Level */
  display:none;
  float:left;
  height:auto;
  left:10px;
  margin:0;
  position:absolute;
  top:18px;
  width:200px;
  z-index:100;
}
#main_menu ul li ul li{
  background:#a2a2a2;
  border-bottom:1px solid #878787;
  clear:left;
  float:left;
  margin:0;
  width:100%;
}
#main_menu ul li ul li a{
  clear:left;
  float:left;
  margin:0;
  padding:4px 6px;
  text-align:left;
  width:188px;
}
#main_menu ul li ul li a span {
  background: url('images/sprite-main.png') no-repeat top left;
  height:auto;
  top:32%;
}
#main_menu ul li ul.config_links li a{
  padding:3px 6px;
}
#main_menu ul li ul li:hover,
#main_menu ul li ul li ul li:hover {
  background:#ccc;
  color:#333;
}
#main_menu ul li ul li:hover ul li a {
  color:#fff;
}
#main_menu ul li ul li:hover a,
#main_menu ul li ul li ul li:hover a {
  text-decoration:none;
  color:#333;
}
#main_menu ul li ul li ul{   /* Third Level */
  left:200px;
  position:absolute;
  top:0;
}
#main_menu ul li ul li ul li{
  width:188px;
}
/*
#main_menu.lang-nl ul li ul li ul li,
#main_menu.lang-fr ul li ul li ul li,
#main_menu.lang-sv ul li ul li ul li{
  width:250px;
}
#main_menu.lang-de ul li ul li ul li{
  width:280px;
}
*/
#main_menu ul li ul li#processingContainer ul li{
  padding:4px 6px;
}
#main_menu ul li ul#navaddnzb{
  width:275px;
}
#main_menu ul li ul#navaddnzb li#navupload,
#main_menu ul li ul li#navfetch{
  padding:10px;
  width:255px;
}
#main_menu ul li ul#navaddnzb li#processingContainer a{
  width:255px;
}
#main_menu ul li ul#navaddnzb li#processingContainer ul{
  left:275px;
}
#main_menu ul li ul li#processingContainer ul li label{
  float:left;
  text-align:left;
  width:40%;
}
#main_menu ul li ul li#processingContainer ul li select{
  width:60%;
}
#main_menu ul li ul#navqueue{
  width:230px;
}
#main_menu ul li ul#navqueue li#navqueueoption {
  padding:10px;
  width:210px;
}
#main_menu ul li ul#navqueue li a{
  width:210px;
}
#main_menu ul li ul#navqueue li ul{
  left:230px;
  width:200px;
}
#main_menu ul li ul#navqueue li ul li a{
  width:180px;
}
#main_menu ul li ul#last_warning_link li{
  min-width: 460px;
}
#main_menu ul li ul#last_warning_link li a{
  min-width: 440px;
}

/*
#main_menu.lang-fr #addID_input{
  width: 110px;
}
*/

#addID{
  color:#000;
}

/*
.sf-shadow ul {  if superfish shadows are enabled lets make it happen!
  background:  url('images/shadow.png') no-repeat bottom right;
  padding: 0 8px 9px 0;
  -moz-border-radius-bottomleft: 17px;
  -moz-border-radius-topright: 17px;
  -webkit-border-top-right-radius: 17px;
  -webkit-border-bottom-left-radius: 17px;
}
*/

#uploadTip {
  overflow: hidden;
}

/*
  ColorBox Core Style
  The following rules are the styles that are consistant between themes.
  Avoid changing this area to maintain compatability with future versions of ColorBox.
*/
#colorbox, #cboxOverlay, #cboxWrapper{position:absolute; top:0; left:0; z-index:9999; overflow:hidden;}
#cboxOverlay{position:fixed; width:100%; height:100%;}
#cboxMiddleLeft, #cboxBottomLeft{clear:left;}
#cboxContent{position:relative; overflow:hidden;}
#cboxLoadedContent{overflow:auto;}
#cboxLoadedContent iframe{display:block; width:100%; height:100%; border:0;}
#cboxTitle{margin:0;}
#cboxLoadingOverlay, #cboxLoadingGraphic{position:absolute; top:0; left:0; width:100%;}
#cboxPrevious, #cboxNext, #cboxClose, #cboxSlideshow{cursor:pointer;}

/*
  Example user style
  The following rules are ordered and tabbed in a way that represents the
  order/nesting of the generated HTML, so that the structure easier to understand.
*/
#cboxOverlay{background-color:#000;}
#colorbox{}
#cboxTopLeft{width:21px; height:21px; background:url(images/colorbox-controls.png) -100px 0 no-repeat;}
#cboxTopRight{width:21px; height:21px; background:url(images/colorbox-controls.png) -129px 0 no-repeat;}
#cboxBottomLeft{width:21px; height:21px; background:url(images/colorbox-controls.png) -100px -29px no-repeat;}
#cboxBottomRight{width:21px; height:21px; background:url(images/colorbox-controls.png) -129px -29px no-repeat;}
#cboxMiddleLeft{width:21px; background:url(images/colorbox-controls.png) left top repeat-y;}
#cboxMiddleRight{width:21px; background:url(images/colorbox-controls.png) right top repeat-y;}
#cboxTopCenter{height:21px; background:url(images/colorbox-border.png) 0 0 repeat-x;}
#cboxBottomCenter{height:21px; background:url(images/colorbox-border.png) 0 -29px repeat-x;}
#cboxContent{background:#fff;}
#cboxLoadedContent{margin-top:28px;}
#cboxLoadedContent a{font-weight:bold;}
#cboxTitle{position:absolute; top:4px; left:0; text-align:center; width:100%; color:#949494; font-weight:bold;}
#cboxCurrent{position:absolute; top:4px; left:58px; color:#fff;}
#cboxSlideshow{position:absolute; top:4px; right:30px; color:#0092ef;}
#cboxPrevious{position:absolute; top:0; left:0px; background:url(images/colorbox-controls.png) -75px 0px no-repeat; width:25px; height:25px; text-indent:-9999px;}
#cboxPrevious.hover{background-position:-75px -25px;}
#cboxNext{position:absolute; top:0; left:27px; background:url(images/colorbox-controls.png) -50px 0px no-repeat; width:25px; height:25px; text-indent:-9999px;}
#cboxNext.hover{background-position:-50px -25px;}
#cboxClose{position:absolute; top:0; right:0; background:url(images/colorbox-controls.png) -25px 0px no-repeat; width:25px; height:25px; text-indent:-9999px;}
#cboxClose.hover{background-position:-25px -25px;}

#plush_options_modal, #add_nzb_modal {
  padding: 20px 0 0 50px;
}
#plush_options_modal .juiButton, #add_nzb_modal .juiButton {
  color:#3383bb;
}

#help_modal {
  text-align:center;
  padding:20px;
}
#help_modal table {
  margin:0 auto 20px;
  border-spacing:4px;
  font-size:110%;
}
#help_modal .sabnzbd_logo {
  float:none;
  display:block;
  margin:auto;
  padding:0px;
  width:100px;
}

/* ---------------- N Z O (nzb files page, linked from queue) -------------------- */
body.nzo {}
body.nzo .config_nav {
  cursor: default;
}
body.nzo #config_content input,
body.nzo .config_nav a {
  cursor: pointer;
}

table.nzoTable {
  border-collapse:collapse;
  color: #131313;
  margin:1px;
  width:99%;
}

table.nzoTable tr {
  cursor: pointer;
}
table.nzoTable tr.even {
  background-color: #cdf;
}
table.nzoTable tr.evenLine {
  background-color: #eee;
}
table.nzoTable tr.odd {
  background-color: #99bbff;
}
table.nzoTable tr.oddLine {
  background-color: #f8f8f8;
}
table.nzoTable tr.nzo_highlight {
  background-color:#FFFFE0;
}
table.nzoTable tr:hover {
  background-color: #666;
  color: #FFF;
}

table.nzoTable th {
  background: #b5b5b5;
  color: white;
  cursor: default;
  font-weight: bold;
}

table.nzoTable th,
table.nzoTable td {
  border:1px solid #999;
  padding:3px 5px;
}
table.nzoTable td.alignRight {
  text-align:right;
}
table.nzoTable td.filename {
  width:95%;
}
table.nzoTable th.check,
table.nzoTable td.check {
  width:22px;
}

.nzo_save_form { margin-left: 120px; }

.nzo_save_form div {
  display: inline;
  white-space: nowrap;
}

/* --------------------- CONFIG --------------------- */

#config_container {
  background: #363636;
  font-size:95%;
  min-height: 345px;
  padding-left: 20px;
  padding: 10px;
  text-align: left;
  border-radius: 10px;
  -moz-border-radius: 10px;
  -webkit-border-radius: 10px;
}
#config_content {
  overflow: hidden;
  width: 100%;
}
#config_content .catheaderinfo { font-weight:bold;}
#config_content div.tabs {
  min-height:338px;
  overflow:auto;
}

#config_go_back {
  padding: 1px;
  text-align: center;
}

#config_go_back div {
  margin-bottom: -4px;
  margin: auto;
  padding: 25px 0 4px 20px;
  width: 80px;
}

#config_go_back a {
  color: black;
  font-size: 66%;
  margin-left: -20px;
  padding: 30px 10px 0px 40px;
  text-decoration: none;
}

#config_go_back a:hover { border-bottom: 1px dotted black; }

div.EntryBlock form { display: inline; }
div.EntryBlock { }

fieldset.EntryFieldSet {
  border:1px solid #333;
  display: inline;
  float: left;
  margin: 5px;
  padding: 7px 7px 10px;
  vertical-align: top;
}
.add-rss fieldset.EntryFieldSet {
  width:97%;
}
.add-rss fieldset.EntryFieldSet .feedname{
  font-weight:bold;
  width:20%;
}
.add-rss fieldset.EntryFieldSet .feeduri{
  width:70%;
}
.add-rss fieldset.EntryFieldSet .inputsubmit{
  width:6%;
}
.update-rss fieldset.EntryFieldSet .feeduri{
  width:99%;
}
.catTable {
  width:99%;
}
.catTable th,
.catTable td {
  padding:3px 3px 5px;
}
.catTable th {
  font-weight:bold;
}
.catTable tr.odd {
  background-color:#ddd;
}
.catTable td.lastCol {
  width:8%;
}

.rssTable { border-spacing: 1px; }
.rssTable th,
.rssTable td {
  padding:3px 3px 5px;
}
.rssTable th {
  background-color:#555;
  color:#FFF;
  font-weight:bold;
}
.rssTable tr {
  background-color:#eee;
}
.rssTable tr.odd {
  background-color:#ddd;
}

.rssLeftColumn {text-align: right;}

/* Vertical Tabs - adapted from http://jquery-ui.googlecode.com/svn/trunk/demos/tabs/vertical.html
----------------------------------
.ui-tabs-vertical { width: 100%; }
.ui-tabs-vertical .ui-tabs-nav { padding: .2em .1em .2em .2em; float: left; width: 11em; margin-right:10px;}
.ui-tabs-vertical .ui-tabs-nav li { clear: left; width: 100%; border-bottom-width: 1px !important; border-right-width: 0 !important; margin: 0 -1px .2em 0; }
.ui-tabs-vertical .ui-tabs-nav li a { display:block; width:12em; }
.ui-tabs-vertical .ui-tabs-nav li.ui-tabs-selected { padding-bottom: 0; padding-right: .1em; border-right-width: 1px; border-right-width: 1px; }
.ui-tabs-vertical .ui-tabs-panel { padding: 1em; }
*/


#schedulingTable td, #switchesTable td {
  padding:3px 3px 5px;
}
.alignRight {
  text-align: right;
  padding-right: 4px;
}

.config_nav {
  float: left;
  margin-top: 20px;
  text-align: left;
  width: 84px;
  position: fixed;
}
/*
.config_nav.lang-de {
  width: 140px;
}
*/
.config_nav ul {
  border: none;
  list-style: none;
  margin: 0;
  padding: 0;
}

.config_nav li {
  margin: 0;
  padding: 3px 0px 3px 5px;
}

.config_nav li a {
  background-color: #b1b1b1;
  color: #363636;
  display: block;
  font-size: 9pt;
  font-weight: bold;
  padding: 4px 22px 4px 4px;
  text-decoration: none;
  width: 100%;
  border-radius: 10px 0px 0px 10px;
  -moz-border-radius: 10px 0px 0px 10px;
  -webkit-border-top-right-radius: 0px;
  -webkit-border-bottom-right-radius: 0px;
}

.config_nav li a .config_sprite_container {
  margin: 3px 0px 3px 0px;
  padding-left: 22px;
  padding-bottom: 1px;
  width: 100%;
}

.config_nav li:hover a, .config_nav li a:hover {
  background-color: #ccc;
  color: #000;
}

.config_nav .nav_active {
  background-color: #363636;
  color: #fff;
}

legend .config_sprite_container {
  display: inline;
  padding: 0px 0px 0px 16px;
}

#config_content legend {
  background-color: #333;
  color: white;
  font-weight: bold;
  padding: 2px 6px 2px 6px;
}

#config_content legend.latestWarnings,
#config_content legend.lastEmail,
#config_content legend.serverBackup { background-color:#00008B; }
#config_content legend.serverOptional { background-color:#009090; }
#config_content legend.serverEnabled { background-color:#006400; }
#config_content legend.serverDisabled { background-color:#8B0000; }
#config_content legend.feedEnabled { background-color:#006400; }
#config_content legend.feedDisabled { background-color:#8B0000; }

/* sprite-config.png */
.config_sprite_container { background: url('images/sprite-config.png') no-repeat top left; }

.sprite_config_connections_disconnect { background-position: 0 0; width: 22px; height: 22px; }
.sprite_config_connections_encrypted { background-position: 0 -32px; width: 16px; height: 16px; }
.sprite_config_connections_unencrypted { background-position: 0 -58px; width: 16px; height: 16px; }
.sprite_config_connections_warnings { background-position: 0 -84px; width: 16px; height: 16px; }
.sprite_config_email_account_settings { background-position: 0 -110px; width: 16px; height: 16px; }
.sprite_config_email_options, .sprite_config_nav_email { background-position: 0 -136px; width: 16px; height: 16px; }
.sprite_config_email_test { background-position: 0 -162px; width: 22px; height: 22px; }
.sprite_config_folders_system { background-position: 0 -194px; width: 16px; height: 16px; }
.sprite_config_folders_user { background-position: 0 -220px; width: 16px; height: 16px; }
.sprite_config_general_restart { background-position: 0 -246px; width: 22px; height: 22px; }
.sprite_config_general_tuning { background-position: 0 -278px; width: 16px; height: 16px; }
.sprite_config_general_web_server { background-position: 0 -304px; width: 16px; height: 16px; }
.sprite_config_help { background-position: 0 -330px; width: 22px; height: 22px; }
.sprite_config_indexsites_account_info { background-position: 0 -362px; width: 16px; height: 16px; }
.sprite_config_indexsites_bookmarks { background-position: 0 -388px; width: 16px; height: 16px; }
.sprite_config_indexsites_processed { background-position: 0 -414px; width: 16px; height: 16px; }
.sprite_config_nav_categories { background-position: 0 -440px; width: 16px; height: 16px; }
.sprite_config_nav_connections { background-position: 0 -466px; width: 16px; height: 16px; }
.sprite_config_nav_folders { background-position: 0 -492px; width: 16px; height: 16px; }
.sprite_config_nav_general { background-position: 0 -518px; width: 16px; height: 16px; }
.sprite_config_nav_indexsites { background-position: 0 -544px; width: 16px; height: 16px; }
.sprite_config_nav_rss { background-position: 0 -570px; width: 16px; height: 16px; }
.sprite_config_nav_scheduling, .sprite_config_scheduling_current { background-position: 0 -596px; width: 16px; height: 16px; }
.sprite_config_nav_servers, .sprite_config_servers_add { background-position: 0 -622px; width: 16px; height: 16px; }
.sprite_config_nav_sorting, .sprite_config_sorting_series { background-position: 0 -648px; width: 16px; height: 16px; }
.sprite_config_nav_switches { background-position: 0 -674px; width: 16px; height: 16px; }
.sprite_config_rss_add { background-position: 0 -700px; width: 16px; height: 16px; }
.sprite_config_save { background-position: 0 -726px; width: 22px; height: 22px; }
.sprite_config_scheduling_add { background-position: 0 -758px; width: 16px; height: 16px; }
.sprite_config_sorting_date { background-position: 0 -784px; width: 16px; height: 16px; }
.sprite_config_sorting_generic { background-position: 0 -810px; width: 16px; height: 16px; }
.sprite_config_switches_other { background-position: 0 -836px; width: 16px; height: 16px; }
.sprite_config_switches_processing { background-position: 0 -862px; width: 16px; height: 16px; }
.sprite_nzo_bottom { background-position: 0 -888px; width: 16px; height: 16px; }
.sprite_nzo_delete { background-position: 0 -914px; width: 16px; height: 16px; }
.sprite_nzo_down { background-position: 0 -940px; width: 16px; height: 16px; }
.sprite_nzo_reload { background-position: 0 -966px; width: 16px; height: 16px; }
.sprite_nzo_select_all { background-position: 0 -992px; width: 16px; height: 16px; }
.sprite_nzo_select_invert { background-position: 0 -1018px; width: 16px; height: 16px; }
.sprite_nzo_select_none { background-position: 0 -1044px; width: 16px; height: 16px; }
.sprite_nzo_select_range { background-position: 0 -1070px; width: 16px; height: 16px; }
.sprite_nzo_top { background-position: 0 -1096px; width: 16px; height: 16px; }
.sprite_nzo_up { background-position: 0 -1122px; width: 16px; height: 16px; }

.numbered_list li { list-style-type: decimal; margin-left:3em; }

/* jqueryui config */
.ui-tabs .ui-tabs-nav li a {
  font-size:95%;
}

.tabs div {
  text-align:left;
}

/* FF2 to rescue */
.sucker {
  font-weight: bold;
  width: 120px;
}

.tucker {
  font-weight: bold;
  width: 300px;
}

.left_links .config_sprite_container, .right_links .config_sprite_container {
  display: inline;
  margin: 3px 0px 3px 0px;
  padding: 4px 0px 4px 22px;
}

.left_links a, .right_links a {
  cursor: pointer;
  padding: 8px;
  border-radius: 10px;
  -moz-border-radius: 10px;
  -webkit-border-radius: 10px;
}

.left_links {
  float: left;
  margin: 5px 0 0 10px;
}

.left_links a {
  background-color: #363636;
  color: #ddd;
  display: inline;
  font-size: 9pt;
  font-weight: bold;
  text-decoration: none;
}

.left_links a:hover {
  background-color: #000;
  color: #fff;
}

.config_margin { margin: 5px 110px 0 110px; margin-right:10px; }
/*
.config_margin.lang-de { margin: 5px 110px 0 140px; }
*/
.config_margin_wide { }
.config_margin_status { margin: 5px 10px 0 10px; }

.right_links {
  float: right;
  margin: 5px 40px 0 0;
}

.right_links a {
  background-color: #ddd;
  color: #666;
  display: block;
  font-size: 9pt;
  font-weight: bold;
  text-decoration: none;
}

.right_links a:hover {
  background-color: #666;
  color: #fff;
}

.center { text-align: center; }

.xsmall { font-size: x-small; }

.config_body { background: #b1b1b1; }

#configbox_latest_warnings a { color: white; }

.dir_helper {
  background-color: blue;
  color: white;
  padding:2px;
}

.manual_restart {
  background-color: red;
  color: white;
  padding:2px;
}

.optional {
  background-color: green;
  color: white;
}

.key_toggle {
  background-color: #333;
  color: white;
  cursor: pointer;
}

*+html {
  overflow-x:visible;
}

/*
  new config design, adapted from Sickbeard
*/
#config  { padding: 0px; text-align: center; }
#config * { margin:0; font-family: "Trebuchet MS", Verdana, sans-serif; }
#config ul { list-style-type:none; }
#config h3 { font-size:1.5em; color:#000; }
#config h4 { font-size:1em; color:#333; text-transform: uppercase; margin:0 0 .2em; }
#config h5 { font-size:1em; color:#000; margin:0 0 .2em; }
#config p, #config li { font-size:1.2em; line-height:1.3; }

#config .path { font-size:1em; color:#333; font-family: Verdana;}
#config .jumbo {font-size: 16px !important;}

#config-content {
  display: block; width: 100%; text-align: left; padding:0 0 40px; clear: both; background: #fff; margin: 0px auto;
  background: #ebebeb;
  -webkit-border-radius: 4px;
  -moz-border-radius: 4px;
  min-height: 305px;
  overflow:auto;
}
input.ui-button { padding: .2em .5em; }

#config-components { width: auto; }
#config-components-border { float: left; width: auto; border-top: 1px solid #999999; padding: 5px 0;}

#config .component-group { border-bottom: 1px dotted #666; padding: 15px 15px 25px; position: relative; }
#config .component-group-desc { float: left; width: 23%;}
#config .component-group-desc h3 { font-size: 1.5em;  }
#config .component-group-desc p { margin: .8em 0; width: 85%; font-size: 1.2em; color: #666;}
#config .component-group-desc p.note { margin: .8em 0; width: 90%; font-size: 1.2em; color: #333;}
#config .component-group-list { margin-left:25%; width: 75%; }
#config .component-group-last { border-bottom: 0px; padding: 15px 15px 25px; position: relative; }

#config fieldset {border:0; outline: 0;  }

#config div.field-pair { padding: .5em;  }
#config div.field-pair.alt { background-color:#ddd;  }
#config div.field-pair input { float: left; }
#config input[type='text'] { padding-left: 5px; height: 1.7em; }

#config label.nocheck { padding-left: 13px; }
#config label span.component-title { font-size: 1.2em; font-weight: bold; float: left; width: 25%; margin-right: 10px;}
#config label span.component-desc { font-size: 1.0em; font-style:italic;  }

#config div.nocheck { padding-left: 13px; }
#config div span.component-title { font-size: 1.2em; font-weight: bold; float: left; width: 25%; margin-left: 10px; margin-right: 10px;}
#config div span.component-desc { font-size: 1.0em; font-style:italic; width: 75%; }

#config div.field-pair select { font-size: 1.1em; border: 1px solid #d4d0c8;}
#config div.field-pair select option { padding: 0 10px; line-height: 1.4; }

#config-settings { float: right; width: 200px; background: #fffae5; border-bottom: 1px dotted #666; border-top: 1px solid #999;padding: 20px 0 30px; margin-right: 20px; }
#config-settings .config-settings-group { margin: 0 15px 15px; border-bottom: 1px dotted #999; padding-bottom: 15px; }
#config-settings .config-settings-group h2 { margin-bottom: 0; }
#config-settings .config-settings-group p {font-size: 1.1em; color: #666; margin: .6em 0 1em; }
#config-settings .config-settings-group div.field-pair { margin: 1.2em 0 .6em; }
#config-settings .config-settings-group input {float: left;}

.clearfix:after { content: "."; display: block; height: 0; clear: both; visibility: hidden; }
.clearfix { display: inline-block; }
/* Hides from IE-mac \*/
* html .clearfix { height: 1%; }
.clearfix { display: block; }
/* End hide from IE-mac */

.multiple_cats {
  padding: 5px 10px;
}
.apikey {
  width:220px;
}

