# This file will be patched by setup.py
# The __version__ should be set to the branch name
# (e.g. "trunk" or "0.4.x")

# You MUST use double quotes (so " and not ')

__version__ = "1.0.3"
__baseline__ = "927ba3cd9d330a74fbdfc8d926f9cbc6aee764af"
